@@ -14,9 +14,11 @@ extensions = [
     ]
 
 intersphinx_mapping = {
-    'http://docs.python.org': None,
-    'http://django.readthedocs.org/en/latest/': None,
-    'http://sybil.readthedocs.io/en/latest/': None,
+    'https://docs.python.org/3/': None,
+    'https://django.readthedocs.io/en/latest/': None,
+    'https://docs.pytest.org/en/latest/': None,
+    'https://sybil.readthedocs.io/en/latest/': None,
+    'https://zopecomponent.readthedocs.io/en/latest/': None,
 }
 
 # General
@@ -29,10 +31,7 @@ exclude_trees = ['_build']
 pygments_style = 'sphinx'
 
 # Options for HTML output
-if on_rtd:
-    html_theme = 'default'
-else:
-    html_theme = 'classic'
+html_theme = 'furo'
 htmlhelp_basename = project+'doc'
 
 # Options for LaTeX output
@@ -40,3 +39,5 @@ latex_documents = [
     ('index', project+'.tex', project+u' Documentation',
      'Simplistix Ltd', 'manual'),
 ]
+
+exclude_patterns = ['**/furo.js.LICENSE.txt']
