@@ -1,4 +1,5 @@
 import re
+from abc import ABC
 from datetime import date, datetime
 from decimal import Decimal
 
@@ -18,11 +19,7 @@ from testfixtures import (
     generator,
     singleton,
     )
-from testfixtures.compat import (
-    class_type_name, exception_module, PY3, xrange,
-    BytesLiteral, UnicodeLiteral,
-    PY2, PY_37_PLUS, ABC
-)
+from testfixtures.compat import PY_37_PLUS
 from testfixtures.comparison import compare_sequence, compare_object
 from unittest import TestCase
 
@@ -48,17 +45,12 @@ class Lazy:
         return self.message
 
 
-def check_raises(x=marker, y=marker, message=None, regex=None,
+def check_raises(x_=marker, y_=marker, message=None, regex=None,
                  compare=compare, **kw):
     args = []
-    for value in x, y:
+    for value in x_, y_:
         if value is not marker:
             args.append(value)
-    for value in 'x', 'y':
-        explicit = 'explicit_{}'.format(value)
-        if explicit in kw:
-            kw[value] = kw[explicit]
-            del kw[explicit]
     try:
         compare(*args, **kw)
     except Exception as e:
@@ -117,51 +109,35 @@ class TestCompare(CompareHelper, TestCase):
         compare('x', 'x')
 
     def test_unicode_string_different(self):
-        if PY2:
-            expected = "u'a' != 'b'"
-        else:
-            expected = "'a' != b'b'"
-        self.check_raises(
-            UnicodeLiteral('a'), BytesLiteral('b'),
-            expected
-            )
+        self.check_raises('a', b'b', "'a' != b'b'")
 
     def test_bytes_different(self):
-        if PY2:
-            expected = (
-                "\n" 
-                "'12345678901'\n"
-                '!=\n'
-                "'12345678902'"
-            )
-        else:
-            expected = (
-                "\n" 
-                "b'12345678901'\n"
-                '!=\n'
-                "b'12345678902'"
-            )
+        expected = (
+            "\n"
+            "b'12345678901'\n"
+            '!=\n'
+            "b'12345678902'"
+        )
         self.check_raises(
-            BytesLiteral('12345678901'),
-            BytesLiteral('12345678902'),
+            b'12345678901',
+            b'12345678902',
             expected
             )
 
     def test_bytes_same_strict(self):
         compare(actual=b'', expected=b'', strict=True)
 
-    if PY3:
-        def test_moar_bytes_different(self):
-            self.check_raises(
-                actual=b'{"byte_pound":"b\'\\\\xa3\'"}',
-                expected=b'{"byte_pound":"b\\\'\\xa3\'"}',
-                message = (
-                    "\n"
-                    "b'{\"byte_pound\":\"b\\\\\\'\\\\xa3\\\'\"}' (expected)\n"
-                    '!=\n'
-                    "b'{\"byte_pound\":\"b\\\'\\\\\\\\xa3\\\'\"}' (actual)"
-                )
+    def test_moar_bytes_different(self):
+        self.check_raises(
+            actual=b'{"byte_pound":"b\'\\\\xa3\'"}',
+            expected=b'{"byte_pound":"b\\\'\\xa3\'"}',
+            message = (
+                "\n"
+                "b'{\"byte_pound\":\"b\\\\\\'\\\\xa3\\\'\"}' (expected)\n"
+                '!=\n'
+                "b'{\"byte_pound\":\"b\\\'\\\\\\\\xa3\\\'\"}' (actual)"
             )
+        )
 
     def test_string_diff_short(self):
         self.check_raises(
@@ -245,14 +221,13 @@ class TestCompare(CompareHelper, TestCase):
         self.check_raises(
             C(e1), e2,
             ("\n"
-             "<C:{module}.ValueError(failed)>\n"
+             "<C:builtins.ValueError(failed)>\n"
              "attributes differ:\n"
              "'args': ('some message',) (Comparison) "
              "!= ('some other message',) (actual)\n"
-             "</C:{module}.ValueError>"
+             "</C:builtins.ValueError>"
              " != ValueError('some other message'{message})"
-             ).format(module=exception_module,
-                      message='' if PY_37_PLUS else ','))
+             ).format(message='' if PY_37_PLUS else ','))
 
     def test_sequence_long(self):
         self.check_raises(
@@ -420,6 +395,27 @@ class TestCompare(CompareHelper, TestCase):
             "'y': 1.0 != 2.0"
             )
 
+    def test_dict_identical_none_matching_nones_and_ones(self):
+        self.check_raises(
+            {
+                'foo': None,
+                'baz': None,
+            },
+            {
+                'foo': 1,
+                'baz': 1,
+            },
+            "dict not as expected:\n"
+            "\n"
+            'values differ:\n'
+            "'baz': None != 1\n"
+            "'foo': None != 1\n"
+            '\n'
+            "While comparing ['baz']: None != 1\n"
+            "\n"
+            "While comparing ['foo']: None != 1"
+            )
+
     def test_dict_labels_specified(self):
         self.check_raises(
             dict(x=1, y=2), dict(x=2, z=3),
@@ -491,18 +487,13 @@ class TestCompare(CompareHelper, TestCase):
             )
 
     def test_dict_consistent_ordering_types_same(self):
-        if PY3:
-            same = "[6, None]\n"
-        else:
-            same = "[None, 6]\n"
-
         self.check_raises(
             {None: 1, 6: 2, 1: 3},
             {None: 1, 6: 2, 1: 4},
             "dict not as expected:\n"
             "\n"+
             'same:\n'+
-            same+
+            "[6, None]\n"+
             "\n"
             "values differ:\n"
             "1: 3 != 4"
@@ -732,11 +723,11 @@ class TestCompare(CompareHelper, TestCase):
         compare(generator(1, 2, 3), (1, 2, 3))
 
     def test_iterable_with_iterable_same(self):
-        compare(xrange(1, 4), xrange(1, 4))
+        compare(range(1, 4), range(1, 4))
 
     def test_iterable_with_iterable_different(self):
         self.check_raises(
-            xrange(1, 4), xrange(1, 3),
+            range(1, 4), range(1, 3),
             "sequence not as expected:\n"
             "\n"
             "same:\n"
@@ -750,33 +741,29 @@ class TestCompare(CompareHelper, TestCase):
             )
 
     def test_iterable_and_generator(self):
-        compare(xrange(1, 4), generator(1, 2, 3))
+        compare(range(1, 4), generator(1, 2, 3))
 
     def test_iterable_and_generator_strict(self):
         expected = compile(
-            "x?range\(1, 4\) \(<(class|type) 'x?range'>\) != "
+            "range\(1, 4\) \(<(class|type) 'x?range'>\) != "
             "<generator object (generator )?at... "
             "\(<(class|type) 'generator'>\)"
             )
         self.check_raises(
-            xrange(1, 4), generator(1, 2, 3),
+            range(1, 4), generator(1, 2, 3),
             regex=expected,
             strict=True,
             )
 
     def test_generator_and_iterable(self):
-        compare(generator(1, 2, 3), xrange(1, 4))
+        compare(generator(1, 2, 3), range(1, 4))
 
     def test_tuple_and_list(self):
         compare((1, 2, 3), [1, 2, 3])
 
     def test_tuple_and_list_strict(self):
-        if PY2:
-            expected = ("(1, 2, 3) (<type 'tuple'>) != "
-                        "[1, 2, 3] (<type 'list'>)")
-        else:
-            expected = ("(1, 2, 3) (<class 'tuple'>) != "
-                        "[1, 2, 3] (<class 'list'>)")
+        expected = ("(1, 2, 3) (<class 'tuple'>) != "
+                    "[1, 2, 3] (<class 'list'>)")
 
         self.check_raises(
             (1, 2, 3), [1, 2, 3],
@@ -794,21 +781,14 @@ class TestCompare(CompareHelper, TestCase):
             pass
         compare(X, X)
 
-    def test_old_style_classes_different(self):
-        if PY3:
-            expected = (
-                "<class 'testfixtures.tests.test_compare.TestCompare."
-                "test_old_style_classes_different.<locals>.X'>"
-                " != "
-                "<class 'testfixtures.tests.test_compare.TestCompare."
-                "test_old_style_classes_different.<locals>.Y'>"
-                )
-        else:
-            expected = (
-                "<class testfixtures.tests.test_compare.X at ...>"
-                " != "
-                "<class testfixtures.tests.test_compare.Y at ...>"
-                )
+    def test_default_style_classes_different(self):
+        expected = (
+            "<class 'testfixtures.tests.test_compare.TestCompare."
+            "test_default_style_classes_different.<locals>.X'>"
+            " != "
+            "<class 'testfixtures.tests.test_compare.TestCompare."
+            "test_default_style_classes_different.<locals>.Y'>"
+            )
 
         class X:
             pass
@@ -823,20 +803,13 @@ class TestCompare(CompareHelper, TestCase):
         compare(X, X)
 
     def test_new_style_classes_different(self):
-        if PY3:
-            expected = (
-                "<class 'testfixtures.tests.test_compare.TestCompare."
-                "test_new_style_classes_different.<locals>.X'>"
-                " != "
-                "<class 'testfixtures.tests.test_compare.TestCompare."
-                "test_new_style_classes_different.<locals>.Y'>"
-                )
-        else:
-            expected = (
-                "<class 'testfixtures.tests.test_compare.X'>"
-                " != "
-                "<class 'testfixtures.tests.test_compare.Y'>"
-                )
+        expected = (
+            "<class 'testfixtures.tests.test_compare.TestCompare."
+            "test_new_style_classes_different.<locals>.X'>"
+            " != "
+            "<class 'testfixtures.tests.test_compare.TestCompare."
+            "test_new_style_classes_different.<locals>.Y'>"
+            )
 
         class X(object):
             pass
@@ -997,7 +970,7 @@ b
             "second:\n[2]"
             )
 
-    def test_strict_okay(self):
+    def test_same_object_strict_okay(self):
         m = object()
         compare(m, m, strict=True)
 
@@ -1037,8 +1010,8 @@ b
         m.aCall()
         self.check_raises(
             [call.aCall()], m.method_calls,
-            ("[call.aCall()] (<{0} 'list'>) != [call.aCall()] "
-             "({1})").format(class_type_name, call_list_repr),
+            ("[call.aCall()] (<class 'list'>) != [call.aCall()] "
+             f"({call_list_repr})"),
             strict=True,
             )
 
@@ -1048,9 +1021,9 @@ b
         self.check_raises(
             [call.call('Y'*20)], m.method_calls,
             ("[call.call('YYYYYYYYYYYYYYYYYY... "
-             "(<{0} 'list'>) != "
+             "(<class 'list'>) != "
              "[call.call('XXXXXXXXXXXXXXXXXX... "
-             "({1})").format(class_type_name, call_list_repr),
+             f"({call_list_repr})"),
             strict=True,
             )
 
@@ -1382,10 +1355,7 @@ b
             )
 
     def test_strict_nested_different(self):
-        if PY2:
-            expected = "[1, 2] (<type 'list'>) != (1, 3) (<type 'tuple'>)"
-        else:
-            expected = "[1, 2] (<class 'list'>) != (1, 3) (<class 'tuple'>)"
+        expected = "[1, 2] (<class 'list'>) != (1, 3) (<class 'tuple'>)"
 
         self.check_raises(
             (1, 2, [1, 2]), (1, 2, (1, 3)),
@@ -1453,7 +1423,7 @@ b
                           message="'x' (x_label) != 'y' (y_label)")
 
     def test_explicit_and_labels(self):
-        self.check_raises(explicit_x='x', explicit_y='y',
+        self.check_raises(x='x', y='y',
                           x_label='x_label', y_label='y_label',
                           message="'x' (x_label) != 'y' (y_label)")
 
@@ -1589,6 +1559,20 @@ b
         m.foo(1, 2, x=3)
         compare(m.mock_calls, m.mock_calls, strict=True)
 
+    def test_mock_call_equal(self):
+        m1 = Mock()
+        m1.foo(1, 2, x=3)
+        m2 = Mock()
+        m2.foo(1, 2, x=3)
+        compare(m1.mock_calls, m2.mock_calls)
+
+    def test_mock_call_equal_strict(self):
+        m1 = Mock()
+        m1.foo(1, 2, x=3)
+        m2 = Mock()
+        m2.foo(1, 2, x=3)
+        compare(m1.mock_calls, m2.mock_calls, strict=True)
+
     def test_calls_different(self):
         m1 = Mock()
         m2 = Mock()
@@ -1941,8 +1925,6 @@ b
             )
 
     def test_regex(self):
-        if PY2:
-            return
         shared_prefix = "a" * 199
         self.check_raises(
             re.compile(shared_prefix + "x"),
@@ -1950,6 +1932,146 @@ b
             'Both x and y appear as "re.compile(\''+'a'*199+')", but are not equal!'
             )
 
+    def test_self_referential_same(self):
+        expected = {1: 'foo'}
+        expected[2] = expected
+        actual = {1: 'foo'}
+        actual[2] = actual
+        compare(expected, actual)
+
+    def test_self_referential_different(self):
+        expected = {1: 'foo'}
+        expected[2] = expected
+        actual = {1: 'bar'}
+        actual[2] = actual
+        self.check_raises(
+            expected,
+            actual,
+            'dict not as expected:\n'
+            '\n'
+            'same:\n'
+            '[2]\n'
+            '\n'
+            'values differ:\n'
+            "1: 'foo' != 'bar'\n"
+            '\n'
+            "While comparing [1]: 'foo' != 'bar'"
+        )
+
+    def test_self_referential_different_but_shows_already_seen(self):
+        ouroboros1 = {}
+        ouroboros1['ouroboros'] = ouroboros1
+        ouroboros2 = {}
+        ouroboros2['ouroboros'] = ouroboros2
+        id2 = str(id(ouroboros2))
+        self.check_raises(
+            {1: ouroboros1, 2: 'foo'},
+            {1: ouroboros2, 2: ouroboros2},
+            'dict not as expected:\n'
+            '\n'
+            'same:\n'
+            '[1]\n'
+            '\n'
+            'values differ:\n'
+            "2: 'foo' != {'ouroboros': <Recursion on dict with id="+id2+">}\n"
+            '\n'
+            "While comparing [2]: 'foo' != "
+            "<AlreadySeen for {'ouroboros': {...}} at [1] with id "+id2+">"
+        )
+
+    def test_self_referential_object_tree(self):
+
+        class Node:
+
+            def __init__(self):
+                self.parent = None
+                self.children = []
+
+            def add(self, child: 'Node'):
+                self.children.append(child)
+                child.parent = self
+
+            def __repr__(self):
+                return f'<Node: {self.children}>'
+
+        expected = Node()
+        expected.add(Node())
+        expected.add(Node())
+
+        actual = Node()
+        actual.add(Node())
+
+        self.check_raises(
+            expected,
+            actual,
+            'Node not as expected:\n'
+            '\n'
+            'attributes same:\n'
+            "['parent']\n"
+            '\n'
+            'attributes differ:\n'
+            "'children': [<Node: []>, <Node: []>] != [<Node: []>]\n"
+            '\n'
+            'While comparing .children: sequence not as expected:\n'
+            '\n'
+            'same:\n'
+            '[<Node: []>]\n'
+            '\n'
+            'first:\n'
+            '[<Node: []>]\n'
+            '\n'
+            'second:\n'
+            '[]'
+        )
+
+    def test_repeated_object_on_the_left_side_ignore_eq(self):
+        item = [1, 2, 3]
+        compare(expected=[item, item], actual=[[1, 2, 3], [1, 2, 3]], ignore_eq=True)
+
+    def test_repeated_object_on_both_sides_ignore_eq(self):
+        item = [1, 2, 3]
+        compare(expected=[item, item], actual=[item, [1, 2, 3]], ignore_eq=True)
+
+    def test_repeated_object_on_both_sides_left_at_compare_strict_type_same(self):
+        item = [1, 2, 3]
+        compare(expected=[item, item], actual=[item, [1, 2, 3]], strict=True)
+
+    def test_repeated_object_on_both_sides_right_at_compare_strict_type_same(self):
+        item = [1, 2, 3]
+        compare(expected=[item, [1, 2, 3]], actual=[item, item], strict=True)
+
+    def test_repeated_object_on_both_sides_strict_type_different(self):
+        item = [1, 2, 3]
+
+        class MyList(list):
+
+            def __repr__(self):
+                return f'<{type(self).__name__}:{super().__repr__()}>'
+
+        type_repr = repr(MyList)
+
+        self.check_raises(
+            [item, item],
+            [item, MyList((1, 2, 3))],
+            strict=True,
+            message = (
+                'sequence not as expected:\n'
+                '\n'
+                'same:\n'
+                '[[1, 2, 3]]\n'
+                '\n'
+                'first:\n'
+                '[[1, 2, 3]]\n'
+                '\n'
+                'second:\n'
+                '[<MyList:[1, 2, 3]>]\n'
+                '\n'
+                f"While comparing [1]: <AlreadySeen for [1, 2, 3] at ... "
+                f"(<class 'testfixtures.comparison.AlreadySeen'>) != "
+                f"<MyList:[1, 2, 3]> ({type_repr})"
+            )
+        )
+
 
 class TestIgnore(CompareHelper):
 
